/*
Copyright The ORAS Authors.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package option_test

import (
	"testing"

	"github.com/spf13/pflag"
	"oras.land/oras/cmd/oras/internal/option"
)

func (t *Test) ApplyFlags(_ *pflag.FlagSet) {
	*t.CntPtr++
}

func TestApplyFlags(t *testing.T) {
	cnt := 0
	type args struct {
		Test
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{"flags should be applied once", args{Test{CntPtr: &cnt}}, false},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			option.ApplyFlags(&tt.args, nil)
			if cnt != 1 {
				t.Errorf("Expect ApplyFlags() to be called once but got %v", cnt)
			}
		})
	}
}
